library puma_firebase;

import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/material.dart';
import 'package:google_sign_in/google_sign_in.dart';
import 'package:puma_firebase/entities/fireuser.dart';
import 'package:puma_firebase/helpers/signin_helper.dart';
import 'package:sign_in_with_apple/sign_in_with_apple.dart';

// Puma FireBase.
class PumaFirebase {
  final bool pushNotifications;
  final bool authentication;
  final Function() onInitSuccess;
  final Function(Object) onInitError;
  final Function(User)? onUserSignIn;
  final Function()? onUserSignOut;

  PumaFirebase({
    required this.onInitSuccess,
    required this.onInitError,
    this.onUserSignIn,
    this.onUserSignOut,
    this.pushNotifications = false,
    this.authentication = false,
  }) {
    debugPrint("initializeFlutterFire");
    appInit();
  }

  static signOut() {
    FirebaseAuth.instance.signOut();
  }

  static bool isSignedIn() {
    User? user = FirebaseAuth.instance.currentUser;
    if (user == null) {
      return false;
    }

    return true;
  }

  static User? getUser() {
    return FirebaseAuth.instance.currentUser;
  }

  void appInit() async {
    try {
      await Firebase.initializeApp();
      debugPrint("_initialized !");
      if (authentication) {
        authInit();
      }

      if (pushNotifications) {
        messagingInit();
      }

      onInitSuccess();
    } catch (e) {
      onInitError(e);
    }
  }

  void messagingInit() async {
    FirebaseMessaging messaging = FirebaseMessaging.instance;
    await messaging.requestPermission(
      alert: true,
      announcement: false,
      badge: true,
      carPlay: false,
      criticalAlert: false,
      provisional: false,
      sound: true,
    );
    messaging.getToken().then((value) {
      debugPrint("token:$value");
    });
  }

  void authInit() {
    FirebaseAuth.instance.authStateChanges().listen((User? user) {
      if (user == null) {
        debugPrint('User is currently signed out!');
        onUserSignOut!();
        // vm.setFirebaseUserUUID(null);
      } else {
        // FirebaseMessaging messaging = FirebaseMessaging.instance;
        debugPrint('User is signed in!');
        debugPrint("UID ${user.uid}");
        if (pushNotifications) {
          subscribeToNotification(user.uid);
        }
        // messaging.subscribeToTopic('${user.uid}');
        // vm.setFirebaseUserUUID(user.uid);
        // vm.synchronizeFirebaseAction();
        onUserSignIn!(user);
      }
    });
  }

  void subscribeToNotification(String topic) {
    FirebaseMessaging messaging = FirebaseMessaging.instance;
    messaging.subscribeToTopic(topic);
  }

  static void signInWithApple({
    required Function(bool, PumaFireUser?) onCompleted,
    required Function(dynamic) onError,
  }) async {
    // To prevent replay attacks with the credential returned from Apple, we
    // include a nonce in the credential request. When signing in with
    // Firebase, the nonce in the id token returned by Apple, is expected to
    // match the sha256 hash of `rawNonce`.
    final rawNonce = generateNonce();
    final nonce = sha256ofString(rawNonce);
    var completed = false;

    // Request credential for the currently signed in Apple account.
    final appleCredential = await SignInWithApple.getAppleIDCredential(
      scopes: [
        AppleIDAuthorizationScopes.email,
        AppleIDAuthorizationScopes.fullName,
      ],
      nonce: nonce,
    ).catchError((error) {
      // print("Catch error $error");
      onError(error);
    }).whenComplete(() {
      completed = true;
    });

    bool success = false;
    PumaFireUser user = PumaFireUser();

    if (completed) {
      // print("Apple Credentials ${appleCredential.givenName} ${appleCredential.familyName}");
      // Create an `OAuthCredential` from the credential returned by Apple.
      final oauthCredential = OAuthProvider("apple.com").credential(
        idToken: appleCredential.identityToken,
        rawNonce: rawNonce,
      );

      // Sign in the user with Firebase. If the nonce we generated earlier does
      // not match the nonce in `appleCredential.identityToken`, sign in will fail.
      FirebaseAuth.instance.signInWithCredential(oauthCredential).then(
        (firebaseUser) {
          user = PumaFireUser(
            uid: firebaseUser.user?.uid,
            email: appleCredential.email,
            givenName: appleCredential.givenName,
            familyName: appleCredential.familyName,
          );
        },
      );

      success = true;
    }

    onCompleted(success, user);
  }

  static void signInWithGoogle({
    required Function(bool, PumaFireUser?) onCompleted,
    required Function(dynamic) onError,
  }) async {
    var completed = false;
    GoogleSignInAccount? googleUser;

    try {
      // Trigger the authentication flow
      googleUser = await GoogleSignIn().signIn().whenComplete(() {
        completed = true;
        return null;
      }).catchError((onError) {
        // print("Caught error");
        onError(onError);
      });
    } catch (e) {
      googleUser = null;
      onError(e);
    }

    bool success = false;
    PumaFireUser user = PumaFireUser();

    if (googleUser != null) {
      // Obtain the auth details from the request
      final GoogleSignInAuthentication googleAuth =
          await googleUser.authentication;

      if (completed) {
        // Create a new credential
        final credential = GoogleAuthProvider.credential(
          accessToken: googleAuth.accessToken,
          idToken: googleAuth.idToken,
        );

        // Once signed in, return the UserCredential
        FirebaseAuth.instance.signInWithCredential(credential).then(
          (firebaseUser) {
            user = PumaFireUser(
              uid: firebaseUser.user?.uid,
              email: googleUser?.email,
              givenName: googleUser?.displayName,
              avatar: googleUser?.photoUrl,
            );
          },
        );

        success = true;
      }
    }

    onCompleted(success, user);
  }
}
